@@ -19,7 +19,7 @@ from sortedcontainers import SortedList, SortedListWithKey
 
 
 __all__ = ['Gauge', 'Momentum']
-__version__ = '0.1.4'
+__version__ = '0.1.5'
 
 
 # indices
@@ -39,6 +39,7 @@ def deprecate(message, *args, **kwargs):
 
 
 def now_or(time):
+    """Returns the current time if `time` is ``None``."""
     return now() if time is None else float(time)
 
 
@@ -211,10 +212,11 @@ class Gauge(object):
         except IndexError:
             return (determination[-1][VALUE], 0.)
         prev_time, prev_value = determination[x - 1]
-        time = float(at - prev_time) / (next_time - prev_time)
-        delta = next_value - prev_value
-        value = prev_value + time * delta
-        velocity = delta / (next_time - prev_time)
+        time_delta = next_time - prev_time
+        value_delta = next_value - prev_value
+        ratio = float(at - prev_time) / time_delta
+        value = prev_value + ratio * value_delta
+        velocity = value_delta / time_delta
         return (value, velocity)
 
     def get(self, at=None):
@@ -477,78 +479,102 @@ class Gauge(object):
         ceil = Boundary(self.walk_segs(self.max), operator.lt)
         floor = Boundary(self.walk_segs(self.min), operator.gt)
         boundaries = [ceil, floor]
-        # skip past boundaries.
+        def clamp_by_boundaries(value, at):
+            if bound is None or overlapped:
+                value = min(value, ceil.seg.guess(at))
+                value = max(value, floor.seg.guess(at))
+            return value
         for boundary in boundaries:
+            # skip past boundaries.
             while boundary.seg.until <= since:
                 boundary.walk()
+            # check overflowing.
+            if bound is not None:
+                continue
+            boundary_value = boundary.seg.guess(since)
+            if boundary.cmp(boundary_value, value):
+                bound, overlapped = boundary, False
         for time, method, momentum in self.walk_events():
             # normalize time.
             until = max(time, self.base[TIME])
-            # check if the value is out of bound.
-            if bound is None:
-                for boundary in boundaries:
-                    boundary_value = boundary.seg.guess(since)
-                    if boundary.cmp(value, boundary_value):
-                        continue
-                    bound = boundary
-                    overlapped = value == boundary_value
-                    break
-            elif bound.cmp_inv(bound.seg.velocity, velocity):
-                overlapped = True
             # if True, An iteration doesn't choose next boundaries.  The first
             # iteration doesn't require to choose next boundaries.
             again = True
             while since < until:
                 if again:
                     again = False
+                    walked_boundaries = boundaries
                 else:
                     # stop the loop if all boundaries have been proceeded.
                     if all(b.seg.until >= until for b in boundaries):
                         break
                     # choose the next boundary.
                     boundary = min(boundaries, key=lambda b: b.seg.until)
-                    if boundary.seg.until < until:
-                        boundary.walk()
-                if bound is not None and bound.seg.until <= since:
-                    continue
-                # current segment.
+                    boundary.walk()
+                    walked_boundaries = [boundary]
+                # calculate velocity.
                 if bound is None:
                     velocity = sum(velocities)
                 elif overlapped:
                     velocity = bound.best(sum(velocities), bound.seg.velocity)
                 else:
                     velocity = sum(v for v in velocities if bound.cmp(v, 0))
-                seg = Segment(value, velocity, since, until)
                 # is still bound?
                 if overlapped and bound.cmp(velocity, bound.seg.velocity):
                     bound, overlapped = None, False
                     again = True
                     continue
+                # current segment.
+                seg = Segment(value, velocity, since, until)
                 if overlapped:
                     bound_until = min(bound.seg.until, until)
                     if bound_until == +inf:
                         break
                     # released from the boundary.
                     since, value = (bound_until, seg.get(bound_until))
+                    value = clamp_by_boundaries(value, at=since)
                     yield (since, value)
                     continue
-                for boundary in boundaries:
+                for boundary in walked_boundaries:
                     # find the intersection with a boundary.
                     try:
                         intersection = seg.intersection(boundary.seg)
                     except ValueError:
                         continue
-                    if intersection[TIME] == seg.since:
+                    if intersection[TIME] == since:
                         continue
-                    since, value = intersection
-                    bound, overlapped = boundary, True
                     again = True  # iterate with same boundaries again.
+                    bound, overlapped = boundary, True
+                    since, value = intersection
+                    # adjust by more accurate value.
+                    if boundary.seg.velocity == 0:
+                        value = boundary.seg.value
+                    elif boundary.seg.since == since:
+                        value = boundary.seg.value
+                    value = clamp_by_boundaries(value, at=since)
+                    yield (since, value)
+                    break
+                if bound is not None:
+                    continue  # the intersection was found.
+                for boundary in walked_boundaries:
+                    # find missing intersection caused by floating-point
+                    # inaccuracy.
+                    bound_until = min(boundary.seg.until, until)
+                    if bound_until == +inf or bound_until < since:
+                        continue
+                    boundary_value = boundary.seg.get(bound_until)
+                    if boundary.cmp_eq(seg.get(bound_until), boundary_value):
+                        continue
+                    bound, overlapped = boundary, True
+                    since, value = bound_until, boundary_value
+                    value = clamp_by_boundaries(value, at=since)
                     yield (since, value)
                     break
             if until == +inf:
                 break
             # determine the last node in the current itreration.
             value += velocity * (until - since)
+            value = clamp_by_boundaries(value, at=until)
             yield (until, value)
             # prepare the next iteration.
             if method == ADD:
